@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2008 by Eike Hein <hein@kde.org>
+  Copyright (C) 2008-2009 by Eike Hein <hein@kde.org>
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
@@ -28,13 +28,11 @@
 
 #include <QString>
 
-#ifdef Q_WS_X11
+#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
 #include <X11/Xlib.h>
 #include <X11/extensions/Xrender.h>
-#endif
 
 
-#ifdef Q_WS_X11
 void getDisplayInformation(Display*& display, Visual*& visual, Colormap& colormap);
 #endif
 
@@ -44,10 +42,10 @@ int main (int argc, char *argv[])
     KAboutData aboutData("yakuake",
         0,
         ki18nc("@title", "<application>Yakuake</application>"),
-        "2.9.4",
+        "2.9.5",
         ki18nc("@title", "A drop-down terminal emulator based on KDE Konsole technology."),
         KAboutData::License_GPL,
-        ki18nc("@info:credit", "(c) 2008 The Yakuake Team"),
+        ki18nc("@info:credit", "(c) 2008-2009 The Yakuake Team"),
         ki18n(0),
         "http://yakuake.kde.org/");
 
@@ -59,6 +57,8 @@ int main (int argc, char *argv[])
         ki18nc("@info:credit", "Project Founder, Default skin (Inactive)"));
     aboutData.addCredit(ki18nc("@info:credit", "Daniel 'suslik' D."), 
         ki18nc("@info:credit", "Plastik skin"), "dd@accentsolution.com");
+    aboutData.addCredit(ki18nc("@info:credit", "Juan Carlos Torres"),
+        ki18nc("@info:credit", "Tab bar drag and drop support, Prevent Closing toggle"), "carlosdgtorres@gmail.com");
 
     KCmdLineArgs::init(argc, argv, &aboutData);
 
@@ -68,7 +68,7 @@ int main (int argc, char *argv[])
         exit(0);
     }
 
-#ifdef Q_WS_X11
+#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
     if (KWindowSystem::compositingActive()) 
     {
             Display* display = 0;
@@ -91,7 +91,7 @@ int main (int argc, char *argv[])
 }
 
 // Code from the Qt 4 graphics dojo examples at http://labs.trolltech.com
-#ifdef Q_WS_X11
+#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
 void getDisplayInformation(Display*& display, Visual*& visual, Colormap& colormap)
 {
     display = XOpenDisplay(0);
