@@ -1,5 +1,6 @@
 /*
-  Copyright (C) 2008 by Eike Hein <hein@kde.org>
+  Copyright (C) 2008-2009 by Eike Hein <hein@kde.org>
+  Copyright (C) 2009 by Juan Carlos Torres <carlosdgtorres@gmail.com>
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
@@ -47,7 +48,7 @@
 #include <QtDBus/QtDBus>
 #include <QTimer>
 
-#ifdef Q_WS_X11
+#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
 #include <QX11Info>
 
 #include <X11/Xlib.h>
@@ -60,6 +61,10 @@ MainWindow::MainWindow(QWidget* parent)
 {
     QDBusConnection::sessionBus().registerObject("/yakuake/window", this, QDBusConnection::ExportScriptableSlots);
 
+#if KDE_IS_VERSION(4,2,68)
+    setAttribute(Qt::WA_TranslucentBackground, true);
+#endif
+
     m_skin = new Skin();
     m_menu = new KMenu(this);
     m_helpMenu = new KHelpMenu(this, KGlobal::mainComponent().aboutData());
@@ -74,6 +79,9 @@ MainWindow::MainWindow(QWidget* parent)
     connect(m_tabBar, SIGNAL(newTabRequested()), m_sessionStack, SLOT(addSession()));
     connect(m_tabBar, SIGNAL(tabSelected(int)), m_sessionStack, SLOT(raiseSession(int)));
     connect(m_tabBar, SIGNAL(tabClosed(int)), m_sessionStack, SLOT(removeSession(int)));
+    connect(m_tabBar, SIGNAL(requestTerminalHighlight(int)), m_sessionStack, SLOT(handleTerminalHighlightRequest(int)));
+    connect(m_tabBar, SIGNAL(requestRemoveTerminalHighlight()), m_sessionStack, SIGNAL(removeTerminalHighlight()));
+    connect(m_tabBar, SIGNAL(tabContextMenuClosed()), m_sessionStack, SIGNAL(removeTerminalHighlight()));
 
     connect(m_sessionStack, SIGNAL(sessionAdded(int, const QString&)), 
         m_tabBar, SLOT(addTab(int, const QString&)));
@@ -100,6 +108,9 @@ MainWindow::MainWindow(QWidget* parent)
         if (Settings::showPopup()) showStartupPopup();
         if (Settings::pollMouse()) toggleMousePoll(true);
     }
+
+    if (Settings::openAfterStart())
+        QMetaObject::invokeMethod(this, "toggleWindowState", Qt::QueuedConnection);
 }
 
 MainWindow::~MainWindow()
@@ -111,11 +122,28 @@ MainWindow::~MainWindow()
 
 bool MainWindow::queryClose()
 {
-    if (Settings::confirmQuit() && m_sessionStack->count() > 1)
+    bool confirmQuit = Settings::confirmQuit();
+    bool hasUnclosableSessions = m_sessionStack->hasUnclosableSessions();
+
+    QString closeQuestion = i18nc("@info","Are you sure you want to quit?");
+    QString warningMessage;
+
+    if ((confirmQuit && m_sessionStack->count() > 1) || hasUnclosableSessions)
     {
+        if (confirmQuit && m_sessionStack->count() > 1)
+        {
+            if (hasUnclosableSessions)
+                warningMessage = i18nc("@info", "<warning>There are multiple open sessions, <emphasis>some of which you have locked to prevent closing them accidentally.</emphasis> These will be killed if you continue.</warning>");
+            else
+                warningMessage = i18nc("@info", "<warning>There are multiple open sessions. These will be killed if you continue.</warning>");
+        }
+        else if (hasUnclosableSessions)
+        {
+            warningMessage = i18nc("@info", "<warning>There are one or more open sessions that you have locked to prevent closing them accidentally. These will be killed if you continue.</warning>");
+        }
+
         int result = KMessageBox::warningContinueCancel(this,
-            i18nc("@info", "<warning>You have multiple open sessions. These will be killed if you continue.</warning><nl/><nl/>"
-                           "Are you sure you want to quit?"),
+            warningMessage + "<br /><br />" + closeQuestion,
             i18nc("@title:window", "Really Quit?"), KStandardGuiItem::quit(), KStandardGuiItem::cancel());
 
         if (result == KMessageBox::Continue)
@@ -131,19 +159,21 @@ void MainWindow::setupActions()
 {
     m_actionCollection = new KActionCollection(this);
 
+    KToggleFullScreenAction* fullScreenAction = new KToggleFullScreenAction(this);
+    fullScreenAction->setWindow(this);
+    fullScreenAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_F11));
+    m_actionCollection->addAction("view-full-screen", fullScreenAction);
+    connect(fullScreenAction, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
+
     KAction* action = KStandardAction::quit(kapp, SLOT(quit()),actionCollection());
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Q));
 
     action = KStandardAction::aboutApp(m_helpMenu, SLOT(aboutApplication()), actionCollection());
     action = KStandardAction::aboutKDE(m_helpMenu, SLOT(aboutKDE()), actionCollection());
- 
+
     action = KStandardAction::keyBindings(this, SLOT(configureKeys()), actionCollection());
     action = KStandardAction::preferences(this, SLOT(configureApp()), actionCollection());
 
-    action = KStandardAction::fullScreen(this, SLOT(updateFullScreen()), this, actionCollection());
-    action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_F11));
-    connect(action, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
-
     action = KStandardAction::whatsThis(this, SLOT(whatsThis()), actionCollection());
 
     action = actionCollection()->addAction("toggle-window-state");
@@ -165,8 +195,8 @@ void MainWindow::setupActions()
     action = actionCollection()->addAction("edit-profile");
     action->setText(i18nc("@action", "Edit Current Profile..."));
     action->setIcon(KIcon("document-properties"));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
 
     action = actionCollection()->addAction("increase-window-width");
     action->setText(i18nc("@action", "Increase Window Width"));
@@ -213,8 +243,8 @@ void MainWindow::setupActions()
     action->setText(i18nc("@action", "Close Session"));
     action->setIcon(KIcon("tab-close"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_S));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
 
     action = actionCollection()->addAction("previous-session");
     action->setText(i18nc("@action", "Previous Session"));
@@ -232,22 +262,22 @@ void MainWindow::setupActions()
     action->setText(i18nc("@action", "Move Session Left"));
     action->setIcon(KIcon("arrow-left"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Left));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
 
     action = actionCollection()->addAction("move-session-right");
     action->setText(i18nc("@action", "Move Session Right"));
     action->setIcon(KIcon("arrow-right"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Right));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
 
     action = actionCollection()->addAction("rename-session");
     action->setText(i18nc("@action", "Rename Session..."));
     action->setIcon(KIcon("edit-rename"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_S));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
 
     action = actionCollection()->addAction("previous-terminal");
     action->setText(i18nc("@action", "Previous Terminal"));
@@ -265,22 +295,34 @@ void MainWindow::setupActions()
     action->setText(i18nc("@action", "Close Active Terminal"));
     action->setIcon(KIcon("view-close"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_R));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
 
     action = actionCollection()->addAction("split-left-right");
     action->setText(i18nc("@action", "Split Left/Right"));
     action->setIcon(KIcon("view-split-left-right"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_L));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
 
     action = actionCollection()->addAction("split-top-bottom");
     action->setText(i18nc("@action", "Split Top/Bottom"));
     action->setIcon(KIcon("view-split-top-bottom"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_T));
-    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependendAction()));
-    m_contextDependendActions << action;
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
+
+    action = actionCollection()->addAction("toggle-session-keyboard-input");
+    action->setText(i18nc("@action", "Disable Keyboard Input"));
+    action->setCheckable(true);
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(handleContextDependentToggleAction(bool)));
+    m_contextDependentActions << action;
+
+    action = actionCollection()->addAction("toggle-session-prevent-closing");
+    action->setText(i18nc("@action", "Prevent Closing"));
+    action->setCheckable(true);
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(handleContextDependentToggleAction(bool)));
+    m_contextDependentActions << action;
 
     for (uint i = 1; i <= 10; ++i)
     {
@@ -294,7 +336,7 @@ void MainWindow::setupActions()
     m_actionCollection->readSettings();
 }
 
-void MainWindow::handleContextDependendAction(QAction* action, int sessionId)
+void MainWindow::handleContextDependentAction(QAction* action, int sessionId)
 {
     if (sessionId == -1) sessionId = m_sessionStack->activeSessionId();
     if (sessionId == -1) return;
@@ -320,18 +362,44 @@ void MainWindow::handleContextDependendAction(QAction* action, int sessionId)
         m_sessionStack->closeActiveTerminal(sessionId);
 
     if (action == actionCollection()->action("split-left-right"))
-        m_sessionStack->splitLeftRight(sessionId);
+        m_sessionStack->splitSessionLeftRight(sessionId);
 
     if (action == actionCollection()->action("split-top-bottom"))
-        m_sessionStack->splitTopBottom(sessionId);
+        m_sessionStack->splitSessionTopBottom(sessionId);
 }
 
-void MainWindow::setContextDependendActionsQuiet(bool quiet)
+void MainWindow::handleContextDependentToggleAction(bool checked, QAction* action, int sessionId)
 {
-    for (int i = 0; i < m_contextDependendActions.size(); ++i) 
-    {
-        m_contextDependendActions.at(i)->blockSignals(quiet);
-    }
+    if (sessionId == -1) sessionId = m_sessionStack->activeSessionId();
+    if (sessionId == -1) return;
+
+    if (!action) action = qobject_cast<QAction*>(QObject::sender());
+
+    if (action == actionCollection()->action("toggle-session-keyboard-input"))
+        m_sessionStack->setSessionKeyboardInputEnabled(sessionId, !checked);
+
+    if (action == actionCollection()->action("toggle-session-prevent-closing"))
+        m_sessionStack->setSessionClosable(sessionId, !checked);
+}
+
+void MainWindow::setContextDependentActionsQuiet(bool quiet)
+{
+    QListIterator<KAction*> i(m_contextDependentActions);
+
+    while (i.hasNext()) i.next()->blockSignals(quiet);
+}
+
+void MainWindow::handleToggleTerminalKeyboardInput(bool checked)
+{
+    QAction* action = qobject_cast<QAction*>(QObject::sender());
+
+    if (!action || action->data().isNull()) return;
+
+    bool ok = false;
+    int terminalId = action->data().toInt(&ok);
+    if (!ok) return;
+
+    m_sessionStack->setTerminalKeyboardInputEnabled(terminalId, !checked);
 }
 
 void MainWindow::handleSwitchToAction()
@@ -350,7 +418,7 @@ void MainWindow::setupMenu()
     m_menu->addAction(actionCollection()->action(KStandardAction::stdName(KStandardAction::AboutKDE)));
 
     m_menu->addTitle(i18nc("@title:menu", "Quick Options"));
-    m_menu->addAction(actionCollection()->action(KStandardAction::stdName(KStandardAction::FullScreen)));
+    m_menu->addAction(actionCollection()->action("view-full-screen"));
     m_menu->addAction(actionCollection()->action("keep-open"));
 
     m_screenMenu = new KMenu(this);
@@ -405,31 +473,55 @@ void MainWindow::updateWindowSizeMenus()
 
 void MainWindow::updateWindowWidthMenu()
 {
-    QAction* action;
+    QAction* action = 0;
 
-    m_windowWidthMenu->clear(); 
-
-    for (int i = 10; i <= 100; i += 10)
+    if (m_windowWidthMenu->isEmpty())
     {
-        action = m_windowWidthMenu->addAction(QString::number(i) + '%');
-        action->setCheckable(true);
-        action->setData(i);
-        action->setChecked(i == Settings::width());
+        for (int i = 10; i <= 100; i += 10)
+        {
+            action = m_windowWidthMenu->addAction(QString::number(i) + '%');
+            action->setCheckable(true);
+            action->setData(i);
+            action->setChecked(i == Settings::width());
+        }
+    }
+    else
+    {
+        QListIterator<QAction*> i(m_windowWidthMenu->actions());
+
+        while (i.hasNext())
+        {
+            action = i.next();
+
+            action->setChecked(action->data().toInt() == Settings::width());
+        }
     }
 }
 
 void MainWindow::updateWindowHeightMenu()
 {
-    QAction* action;
+    QAction* action = 0;
 
-    m_windowHeightMenu->clear(); 
-
-    for (int i = 10; i <= 100; i += 10)
+    if (m_windowHeightMenu->isEmpty())
     {
-        action = m_windowHeightMenu->addAction(QString::number(i) + '%');
-        action->setCheckable(true);
-        action->setData(i);
-        action->setChecked(i == Settings::height());
+        for (int i = 10; i <= 100; i += 10)
+        {
+            action = m_windowHeightMenu->addAction(QString::number(i) + '%');
+            action->setCheckable(true);
+            action->setData(i);
+            action->setChecked(i == Settings::height());
+        }
+    }
+    else
+    {
+        QListIterator<QAction*> i(m_windowHeightMenu->actions());
+
+        while (i.hasNext())
+        {
+            action = i.next();
+
+            action->setChecked(action->data().toInt() == Settings::height());
+        }
     }
 }
 
@@ -539,10 +631,22 @@ void MainWindow::applyWindowProperties()
 
 void MainWindow::applyWindowGeometry()
 {
-    QAction* action =  actionCollection()->action(KStandardAction::stdName(KStandardAction::FullScreen));
-    if (action->isChecked()) action->activate(KAction::Trigger);
+    int width, height;
 
-    setWindowGeometry(Settings::width(), Settings::height(), Settings::position());
+    QAction* action = actionCollection()->action("view-full-screen");
+
+    if (action->isChecked())
+    {
+        width = 100;
+        height = 100;
+    }
+    else
+    {
+        width = Settings::width();
+        height = Settings::height();
+    }
+
+    setWindowGeometry(width, height, Settings::position());
 }
 
 void MainWindow::setWindowGeometry(int newWidth, int newHeight, int newPosition)
@@ -649,15 +753,19 @@ void MainWindow::paintEvent(QPaintEvent* event)
 
     if (useTranslucency())
     {
+#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
         painter.setCompositionMode(QPainter::CompositionMode_Source);
         painter.fillRect(rect(), Qt::transparent);
+#endif
         painter.setOpacity(qreal(Settings::backgroundColorOpacity()) / 100);
         painter.fillRect(rect(), Settings::backgroundColor());
         painter.setOpacity(1.0);
     }
     else
     {
+#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
         painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
+#endif
         painter.fillRect(rect(), Settings::backgroundColor());
     }
 
@@ -800,14 +908,13 @@ void MainWindow::toggleMousePoll(bool poll)
 void MainWindow::pollMouse()
 {
     QPoint pos = QCursor::pos();
+    QRect workArea = getDesktopGeometry();
 
-    if (pos.y() == 0)
-    {
-        if (Settings::screen() == 0) 
-            toggleWindowState();
-        else if (pos.x() >= getDesktopGeometry().x() && pos.x() <= (getDesktopGeometry().x() + getDesktopGeometry().width()))
-            toggleWindowState();
-    }
+    int windowX = workArea.x() + workArea.width() * Settings::position() * (100 - Settings::width()) / 10000;
+    int windowWidth = workArea.width() * Settings::width() / 100;
+
+    if (pos.y() == 0 && pos.x() >= windowX && pos.x() <= (windowX + windowWidth))
+        toggleWindowState();
 }
 
 void MainWindow::setKeepOpen(bool keepOpen)
@@ -820,23 +927,16 @@ void MainWindow::setKeepOpen(bool keepOpen)
 
 void MainWindow::setFullScreen(bool state)
 {
-     if (state)
-        setWindowGeometry(100, 100, Settings::position());
-     else
-        applyWindowGeometry();
-}
-
-void MainWindow::updateFullScreen()
-{
-    QAction* action =  actionCollection()->action(KStandardAction::stdName(KStandardAction::FullScreen));
-
-    if (action->isChecked())
+    if (state)
     {
+        setWindowState(windowState() | Qt::WindowFullScreen);
         setWindowGeometry(100, 100, Settings::position());
-        this->setWindowState(windowState() | Qt::WindowFullScreen);
     }
     else
-        this->setWindowState(windowState() & ~Qt::WindowFullScreen);
+    {
+        setWindowState(windowState() & ~Qt::WindowFullScreen);
+        setWindowGeometry(Settings::width(), Settings::height(), Settings::position());
+    }
 }
 
 int MainWindow::getScreen()
@@ -849,7 +949,7 @@ int MainWindow::getScreen()
 
 QRect MainWindow::getDesktopGeometry()
 {
-    QAction* action = actionCollection()->action(KStandardAction::stdName(KStandardAction::FullScreen));
+    QAction* action = actionCollection()->action("view-full-screen");
 
     if (action->isChecked()) 
         return KApplication::desktop()->screenGeometry(getScreen());
@@ -876,7 +976,7 @@ void MainWindow::showStartupPopup()
     popup->setTimeout(5000);
     popup->setView(popup->standardView(i18nc("@title:window", 
         "<application>Yakuake</application> Notification"), 
-        i18nc("@info", "Application successfully started!<nl/>"
+        i18nc("@info", "Application successfully started.<nl/>"
                        "Press <shortcut>%1</shortcut> to use it...", shortcut),
         KIconLoader::global()->loadIcon("yakuake", KIconLoader::Small)));
 
@@ -917,7 +1017,7 @@ void MainWindow::firstRunDialogOk()
 
 void MainWindow::updateUseTranslucency()
 {
-#ifdef Q_WS_X11
+#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
     bool ARGB = false;
 
     int screen = QX11Info::appScreen();
@@ -938,6 +1038,12 @@ void MainWindow::updateUseTranslucency()
         m_useTranslucency = KWindowSystem::compositingActive();
     }
     else
+#elif KDE_IS_VERSION(4,2,68)
+    if (Settings::translucency())
+    {
+        m_useTranslucency = KWindowSystem::compositingActive();
+    }
+    else
 #endif
     {
         m_useTranslucency = false;
