@@ -1,5 +1,6 @@
 /*
-  Copyright (C) 2008 by Eike Hein <hein@kde.org>
+  Copyright (C) 2008-2009 by Eike Hein <hein@kde.org>
+  Copyright (C) 2009 by Juan Carlos Torres <carlosdgtorres@gmail.com>
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
@@ -32,6 +33,8 @@ Session::Session(SessionType type, QWidget* parent) : QObject(parent)
 
     m_activeTerminalId = -1;
 
+    m_closable = true;
+
     m_baseSplitter = new Splitter(Qt::Horizontal, parent);
     connect(m_baseSplitter, SIGNAL(destroyed()), this, SLOT(prepareShutdown()));
 
@@ -139,8 +142,10 @@ void Session::setupSession(SessionType type)
 Terminal* Session::addTerminal(QWidget* parent)
 {
     Terminal* terminal = new Terminal(parent);
-    connect(terminal, SIGNAL(titleChanged(int, const QString&)), this, SLOT(setTitle(int, const QString&)));
     connect(terminal, SIGNAL(activated(int)), this, SLOT(setActiveTerminal(int)));
+    connect(terminal, SIGNAL(manuallyActivated(Terminal*)), this, SIGNAL(terminalManuallyActivated(Terminal*)));
+    connect(terminal, SIGNAL(titleChanged(int, const QString&)), this, SLOT(setTitle(int, const QString&)));
+    connect(terminal, SIGNAL(keyboardInputBlocked(Terminal*)), this, SIGNAL(keyboardInputBlocked(Terminal*)));
     connect(terminal, SIGNAL(destroyed(int)), this, SLOT(cleanup(int)));
 
     m_terminals.insert(terminal->id(), terminal);
@@ -155,11 +160,9 @@ void Session::closeTerminal(int terminalId)
 {
     if (terminalId == -1) terminalId = m_activeTerminalId;
     if (terminalId == -1) return;
-    if (!m_terminals.contains(terminalId)) return;    
+    if (!m_terminals.contains(terminalId)) return;
 
-    Terminal* activeTerminal = m_terminals[m_activeTerminalId];
-
-    if (activeTerminal) activeTerminal->deletePart();
+    m_terminals.value(terminalId)->deletePart();
 }
 
 void Session::focusPreviousTerminal()
@@ -226,24 +229,26 @@ void Session::focusNextTerminal()
     }
 }
 
-void Session::splitLeftRight()
+void Session::splitLeftRight(int terminalId)
 {
-    if (m_activeTerminalId == -1) return;
-    if (!m_terminals.contains(m_activeTerminalId)) return;
+    if (terminalId == -1) terminalId = m_activeTerminalId;
+    if (terminalId == -1) return;
+    if (!m_terminals.contains(terminalId)) return;
 
-    Terminal* activeTerminal = m_terminals[m_activeTerminalId];
+    Terminal* terminal = m_terminals.value(terminalId);
 
-    if (activeTerminal) split(activeTerminal, Qt::Horizontal);
+    if (terminal) split(terminal, Qt::Horizontal);
 }
 
-void Session::splitTopBottom()
+void Session::splitTopBottom(int terminalId)
 {
-    if (m_activeTerminalId == -1) return;
-    if (!m_terminals.contains(m_activeTerminalId)) return;
+    if (terminalId == -1) terminalId = m_activeTerminalId;
+    if (terminalId == -1) return;
+    if (!m_terminals.contains(terminalId)) return;
 
-    Terminal* activeTerminal = m_terminals[m_activeTerminalId];
+    Terminal* terminal = m_terminals.value(terminalId);
 
-    if (activeTerminal) split(activeTerminal, Qt::Vertical);
+    if (terminal) split(terminal, Qt::Vertical);
 }
 
 void Session::split(Terminal* terminal, Qt::Orientation orientation)
@@ -303,7 +308,7 @@ void Session::setActiveTerminal(int terminalId)
 {
     m_activeTerminalId = terminalId;
 
-    setTitle(m_activeTerminalId, m_terminals[m_activeTerminalId]->title());
+    setTitle(m_activeTerminalId, m_terminals.value(m_activeTerminalId)->title());
 }
 
 void Session::setTitle(int terminalId, const QString& title)
@@ -319,7 +324,7 @@ void Session::setTitle(int terminalId, const QString& title)
 
 void Session::cleanup(int terminalId)
 {
-    if (m_activeTerminalId == terminalId && m_terminals.size() > 1)
+    if (m_activeTerminalId == terminalId && m_terminals.count() > 1)
         focusPreviousTerminal();
 
     m_terminals.remove(terminalId);
@@ -333,7 +338,7 @@ void Session::cleanup()
 
     m_baseSplitter->recursiveCleanup();
 
-    if (m_terminals.size() == 0) 
+    if (m_terminals.count() == 0)
         m_baseSplitter->deleteLater();
 }
 
@@ -349,33 +354,105 @@ const QString Session::terminalIdList()
     QList<int> keyList = m_terminals.uniqueKeys();
     QStringList idList;
 
-    for (int i = 0; i < keyList.size(); ++i) 
-        idList << QString::number(keyList.at(i));
+    QListIterator<int> i(keyList);
+
+    while (i.hasNext())
+        idList << QString::number(i.next());
 
     return idList.join(",");
 }
 
+bool Session::hasTerminal(int terminalId)
+{
+    return m_terminals.contains(terminalId);
+}
+
+Terminal* Session::getTerminal(int terminalId)
+{
+    if (!m_terminals.contains(terminalId)) return 0;
+
+    return m_terminals.value(terminalId);
+}
+
 void Session::runCommand(const QString& command, int terminalId)
 {
     if (terminalId == -1) terminalId = m_activeTerminalId;
     if (terminalId == -1) return;
     if (!m_terminals.contains(terminalId)) return;
 
-    m_terminals[terminalId]->runCommand(command);
+    m_terminals.value(terminalId)->runCommand(command);
 }
 
 void Session::manageProfiles()
 {
     if ( m_activeTerminalId == -1) return;
-    if (!m_terminals.contains( m_activeTerminalId)) return;
+    if (!m_terminals.contains(m_activeTerminalId)) return;
 
-    m_terminals[m_activeTerminalId]->manageProfiles();
+    m_terminals.value(m_activeTerminalId)->manageProfiles();
 }
 
 void Session::editProfile()
 {
     if ( m_activeTerminalId == -1) return;
-    if (!m_terminals.contains( m_activeTerminalId)) return;
+    if (!m_terminals.contains(m_activeTerminalId)) return;
 
-    m_terminals[m_activeTerminalId]->editProfile();
+    m_terminals.value(m_activeTerminalId)->editProfile();
+}
+
+bool Session::keyboardInputEnabled()
+{
+    int keyboardInputDisabledCount = 0;
+
+    QMapIterator<int, Terminal*> i(m_terminals);
+
+    while (i.hasNext())
+    {
+        i.next();
+
+        if (!i.value()->keyboardInputEnabled())
+            ++keyboardInputDisabledCount;
+    }
+
+    return m_terminals.count() != keyboardInputDisabledCount;
+}
+
+void Session::setKeyboardInputEnabled(bool enabled)
+{
+    QMapIterator<int, Terminal*> i(m_terminals);
+
+    while (i.hasNext())
+    {
+        i.next();
+
+        i.value()->setKeyboardInputEnabled(enabled);
+    }
+}
+
+bool Session::keyboardInputEnabled(int terminalId)
+{
+    if (!m_terminals.contains(terminalId)) return false;
+
+    return m_terminals.value(terminalId)->keyboardInputEnabled();
+}
+
+void Session::setKeyboardInputEnabled(int terminalId, bool enabled)
+{
+    if (!m_terminals.contains(terminalId)) return;
+
+    m_terminals.value(terminalId)->setKeyboardInputEnabled(enabled);
+}
+
+bool Session::hasTerminalsWithKeyboardInputDisabled()
+{
+    QMapIterator<int, Terminal*> i(m_terminals);
+
+    while (i.hasNext())
+    {
+        i.next();
+
+        if (!i.value()->keyboardInputEnabled())
+            return true;
+    }
+
+    return false;
 }
